// Get the power name at a given index for a given type. Used to fill in a list of powers.
//   powers - The list of all powers
//   type - The power we're looking for
//   index - The index of the power.
//  Return: The name of the power found or an empty string 
function getPower(powers, type, index) {
  var count = 0;
  var i = powers.iterator();
  while (i.hasNext()) {
    var power = i.next();
    if ((type == "Utility" && power.get("kind") != "Attack") || 
        (power.get("kind") == "Attack" && power.get("type") == type)) {
      if (count == index) {
        msg = power.get("name");
        if (type == "Utility" && power.get("kind") != "Utility") 
          msg = msg + "(" + power.get("kind") + ")";
        return msg;
      } else {
        count = count + 1;
      } // endif
    } // endif
  } // endwhile
  return "";
}

// Array of equipped items that are searched.
var equippedVars = new Array("armor", "shield", "arms", "feet", "hands", "neck", "leftRing", "rightRing", "waist");

// Get the equipment name at a given index for a given type. Used to fill in a list of equipment.
//   powers - The list of all equipment
//   equipped - Is this a request for equipped items?
//   magic - Is this a request for magic items?
//   type - The type of data we're looking for when looking for weapons or equipped data
//   index - The index of the equipment needed w/in the group.
//  Return: The name of the power found or an empty string 
function getEquipment(equipment, equipped, magic, type, index) {
  
  // Check for equipped items
  if (equipped) {
    if (equipment.containsKey("root.equipped." + type + ".name")) {
      return equipment.get("root.equipped." + type + ".name");
    } // endif
    return "";
  } // endif
  
  // Find other equipment
  var count = 0;
  var line = "";
  var weaponCount = 0;
  var i = equipment.iterator();
  while (i.hasNext()) {
    var e = i.next();
    var isMagic = e.containsKey("itemMagic.name") && e.get("itemMagic.name").length() > 0
    
    // Need to be magic or mundane as requested
    if (magic != isMagic) continue; 
    
    // Make sure this isn't equipped
    var found = false;
    for (var equipped in equippedVars) {
      if (equipment.containsKey("root.equipped." + equippedVars[equipped] + ".name") &&  
          equipment.get("root.equipped." + equippedVars[equipped] + ".name") == e.get("name")) {
        found = true;
        break;
      } // endif
    } // endfor 
    if (found) continue;
    
    // Special work for the weapons
    if (type == "weapon" && e.get("slot") != "Weapon") continue; // Need weapon but it isn't one
    if (type != "weapon" && e.get("slot") == "Weapon" && magic) {
      weaponCount = weaponCount + 1;
      if (weaponCount <= 4) continue; // Remove the first 4 magic weapons
    } // endif
    
    // Must be a valid item, return it or count it
    if (!magic) {
      
      // Too much mundane equipment to allow one item per line, fix that
      if (line.length == 0 || (line.length + e.get("name").length() + 1) < 35) { 
        line = line + (line.length == 0 ? '' : ';') + e.get("name")
        continue;
      } else {
        if (count == index) return line;
        count = count + 1;
        line = "" + e.get("name");
      } // endif
    } else {
      if (count == index) {
        return e.get("name");
      } else {
        count = count + 1;
      } // endif
    } // endif
  } // endif
  return "";
}

/**
 * Get a string containing all the feats
 * 
 * @param {PropertyList} feats Feats to be displayed
 * @return {String} A string containing all of the feats
 */ 
function getFeats(feats) {
  var i = feats.iterator();
  var msg = "";
  while (i.hasNext()) {
    var feat = i.next();
    if (msg.length > 0) msg = msg + "\n";
    msg = msg + feat.get("name");
    var optionValue = String(feat.get("optionValue"));
    if (optionValue != "")
      msg += " (" + optionValue + ")";
  } // endwhile
  return msg;
}

// Get a string containg all the requested type of resistances
//   feats - Feats to be displayed
//   type = type of resistance needed
//  Return: A string containg all of the requested resistances
function getResistances(resistances, type) {
  var msg = "";
  var i = resistances.iterator();
  while (i.hasNext()) {
    var resistance = i.next();
    if (type == resistance.get("type")) {
      if (msg.length > 0) msg = msg + " ";
      msg = msg + resistance.get("keyword");
      if (resistance.get("amount").intValue() > 0) msg = msg + " " + resistance.get("amount");
    } // endif
  } // endwhile
  return msg;
}

/**
 * Get an attack value from a combatant's equipment
 * 
 * @param {PropertyList} equipment The equipment for the combatant
 * @param {Number} index The index of the weapon to fetch.
 * @param {String} value The value to be fetched.
 */
function getAttackInfo(equipment, index, value) {
  
  // Get a weapon, calculate
  var weapon = findWeapon(equipment, index);
  if (weapon == undefined) return "";
  
  // Ability is just the melee attack for a weapon
  if (value == "ability")
    return weapon ? String(weapon.get("name")) : "Unarmed";

  // Proficiency bonus
  var proficiency = weapon ? weapon.get("proficiency").intValue() : 0;
  if (value == "proficiency") return String(proficiency);

  // Type of defense
  if (value == "defense") return "AC";
  
  // Magic item enhancement from level
  var enhancement = weapon.get("level").intValue();
  enhancement = enhancement ? Math.floor((enhancement - 1) / 5) + 1: 0;
  if (value == "enhancement") {
    if (weapon == null) return "";
    if (enhancement == 0) return "";
    return String(enhancement);
  } // endif
  
  // Ability bonus is always strength
  var strength = equipment.get("root.str.currentMod").intValue();
  if (value == "abil") return String(strength);
  
  // Set up the current weapon
  var oldWeapon = equipment.get('root.equipped.weapon');
  var oldOffhandWeapon = equipment.get('root.equipped.offHand');
  var oldCurrent = equipment.get('root.currentWeapon.useOffHandWeapon');
  equipment.put('root.currentWeapon.useOffHandWeapon', false);
  weapon = weapon || new net.rptools.chartool.model.property.SlotPropertyValue("weaponItem", "Unarmed attack");
  if (!(weapon === oldWeapon))
    equipment.put('root.equipped.weapon', weapon);
    
  var power = new net.rptools.chartool.model.property.SlotPropertyValue("power", "Melee Basic Attack");
  power.setParent(equipment.get("root.powers.powers")); // Fake it's parent
  
  // Check for attack or damage
  var ret = "";
  if (value.match("^attack")=="attack") {
    var mod = calculateTotalAttackModifier(equipment.get("root"), power, "primary");
    if (value == "attackMod") ret = String(mod);
    if (value == "attackMisc") ret = String(mod - proficiency - enhancement - strength - equipment.get("root.levelBonus").intValue());
  } else {
    var damage = String(power.get("damage"));
    if (value == "damage") ret = damage.replace(/\s/g, '');
    var totalMods = totalModifiers(equipment.get('root.weapon.normalDamageBonus.modifiers'));
    if (value == "damageMisc") ret = String(totalMods - enhancement);
  } // endif
  
  // Restore the combatant
  if (!(weapon === oldWeapon))
    equipment.put('root.equipped.weapon', oldWeapon);
  if (!(equipment.get('root.equipped.offHand') === oldOffhandWeapon))
    equipment.put('root.equipped.offHand', oldOffhandWeapon);
  equipment.put('root.currentWeapon.useOffHandWeapon', oldCurrent);
  return ret;
}

/**
 * Find a weapon in the combatant's hand or inventory
 * 
 * @param {PropertyList} equipment The equipment for the combatant
 * @param {Number} index The index of the weapon to fetch.
 */
function findWeapon(equipment, index) {

  // Check the equipped weapons first
  var weapon = null;
  var mainHandWeaponName = null;
  var offHandWeaponName = null;
  if (equipment.containsKey("root.equipped.weapon.name") && String(equipment.get("root.equipped.weapon.name")) != '') {
    index -= 1;
    weapon = equipment.get("root.equipped.weapon");
    var mainHandWeaponName = String(equipment.get("root.equipped.weapon.name"));
  } // endif
  if (index >= 0 && equipment.containsKey("root.equipped.offHand.name") && String(equipment.get("root.equipped.offHand.name")) != '') {
    index -= 1;
    weapon = equipment.get("root.equipped.offHand");
    var offHandWeaponName = String(equipment.get("root.equipped.offHand.name"));
  } // endif
  
  // Just check equipment
  var i = equipment.get("items").iterator();
  while (i.hasNext() && index >= 0) {
    var e = i.next();
    if (String(e.get("slot")) != "Weapon" || (mainHandWeaponName && mainHandWeaponName == String(e.get("name")))
        || (offHandWeaponName && offHandWeaponName == String(e.get("name")))) continue;
    weapon = e;
    index -= 1;
  } // endwhile
  
  // If none found, assume unarmed, if not even close then leave blank
  if (index == 0) weapon = null;
  if (index > 0) weapon = undefined;
  return weapon;
}

/**
 * Loop through the modifiers and calculate the straight up bonus minus the die roll.
 * 
 * @param {PropertyList} The list of modifiers to be totaled.
 */
function totalModifiers(modifiers) {
  var Java = new JavaImporter(net.rptools.chartool.model.property, java.util);
  with (Java) {
    var mods = weaponDamageMods(modifiers);
    var t = 0;
    var i = mods.keySet().iterator();
    while (i.hasNext()) {
      var damage = mods.get(i.next());
      if (damage[1] > 0) t = t + damage[1];
    } // endwhile
    return t;
  } // endwith
}  