/**
 * Check to see if the needed proficiency exists.
 * 
 * @param {PropertyList} modifiers The list of available proficiencies for the character
 * @param {String} type Search for this type of proficiency.
 * @param {String?} group Optionally search for this weapon group.
 * @param {String?} name Optionally search for this weapon name.
 */
function isProficient(modifiers, type, group, name) {
  var prof = false;
  if (modifiers && !modifiers.isEmpty() && type != "") {
    var t = String(type).replace(/^\s+|\s+$/g, '').toLowerCase();
    var g = group ? String(group).replace(/^\s+|\s+$/g, '').toLowerCase() : '';
    var n = name ? String(name).replace(/^\s+|\s+$/g, '').toLowerCase() : '';
    var j = modifiers.iterator();
    while (j.hasNext() && !prof) {
      var mod = j.next();
      if (!mod.get("applies").booleanValue()) continue;
      var b = mod.get("bonus").split("\\s*,\\s*");
      for (var i = 0; i < b.length && !prof; i++) {
        var lc = b[i].toLowerCase();
        if (lc == t || (g != "" && lc == g) || (n != "" && lc == n)) prof = true;
      } // endfor 
    } // endwhile
  } // endif
  return prof;
} 

// Apply ongoing damage and remove any start of turn conditions.
//   combatant - combatant being modified.
//  Return: The text to display to the user.
function startTurn(combatant) {
  var Java = new JavaImporter(net.rptools.chartool.model.property, java.util);
  with (Java) {
  
    // Find all ongoing damage in the manual entries, remove duplicates
    var m = "";
    var s = new HashMap();
    var j = combatant.get('conditions.manual').iterator();
    while (j.hasNext()) {
      var c = j.next();
      var n = c.get('name');
      var k = c.get('keyword');
      var a = c.get('amount').intValue();
      if (c.get('duration') == 'Start of Turn') {
        m = m + (m.length > 0 ? '<br/>' : '') + 'Removing ' + n;
        if (k.length > 0 || a != 0)
          m = m + ' (' + (k.length > 0 ? k : '') + (a != 0 ? (k.length > 0 ? ' ' : '') + a : '') + ')';     
        j.remove(); // Remove it, but still considered for damage.
      }
      if (n != 'Ongoing Damage' && n!= 'Regeneration') continue;
      var f = s.get(k);
      if (f != null && f.get('amount').intValue() < c.get('amount')) s.remove(k);
      s.put(k, c);
    } // endwhile

    // Find all ongoing damage in the modifier entries, remove duplicates
    j = combatant.get('conditions.modifiers').listIterator();
    while (j.hasNext()) {
      var mod = j.next();
      if (mod.get('duration') == 'Start of Turn') j.remove(); // Remove it, but still consider it for damage.
      var n = mod.get('type');
      if (!mod.get("applies").booleanValue() || (n != 'Ongoing Damage' && n!= 'Regeneration')) continue;
      var n = mod.get('source');
      var f = s.get(n);
      if (f == null || f.get('amount').intValue() < mod.get('bonus').intValue()) {
        f = new SlotPropertyValue('conditions', 'Ongoing Damage');
        f.put('keyword', n);
        f.put('amount', mod.get('bonus'));
        s.put(n, f);
      } // endif
    } // endwhile
  
    // Apply ongoing damage
    j = s.values().iterator();
    while (j.hasNext()) {
      var c = j.next();
      if (c.get('name') == 'Ongoing Damage') {
        var k = c.get('keyword');
        m = m + (m.length > 0 ? '<br/>' : '') + 'Ongoing ' + (k.length == 0 ? '' : k + ' ') + 'damage: ';
        m = m + enterDamage(combatant.get('hp'), c.get('amount').intValue());
      } else {
        m = m + (m.length > 0 ? '<br/>' : '') + 'Regeneration: ';
        m = m + enterHealing(combatant.get('hp'), c.get('amount').intValue());
      } // endif
    } // endwhile

    // Restore actions
    var m2 = resetActions(combatant);
    if (m2 && m) m = m + '<br/>';
    m = m + m2;
    return m;
  } // endwith
}

// Make saving throws and remove successful saves, also remove end of turn and switch end of next turn
//   combatant - combatant being modified.
//  Return: The text to display to the user.
function endTurn(combatant) {
    
  // Look at manual first
  var j = combatant.get('conditions.manual').iterator();
  var m = "";
  while (j.hasNext()) {
    var c = j.next();
    var n = c.get('name');
    var k = c.get('keyword');
    var a = c.get('amount').intValue();
    var d = c.get('duration');
    if (d == 'End of Turn') {
      m = m + (m.length > 0 ? '<br/>' : '') + 'Removing ' + n;
      if (k.length > 0 || a != 0)
        m = m + ' (' + (k.length > 0 ? k : '') + (a != 0 ? (k.length > 0 ? ' ' : '') + a : '') + ')';     
      j.remove();
      continue;
    } // endif 
    if (d == 'End of Next Turn') {
      c.put('duration', 'End of Turn'); 
      continue;
    } // endif
    if (c.get('duration') == 'Save Ends') {
      m = m + (m.length > 0 ? '<br/>' : '') + 'Making save for ' + c.get('name') + (k.length == 0 ? '' : ' ' + k) + ':';
      var s = makeSave(k, combatant.get('resistances.applied'));
      m = m + s;
      if (s.indexOf('Success!') >= 0) {
        m = m + (m.length > 0 ? '<br/>' : '') + 'Removing ' + n;
        if (k.length > 0 || a != 0)
          m = m + ' (' + (k.length > 0 ? k : '') + (a != 0 ? (k.length > 0 ? ' ' : '') + a : '') + ')';     
        j.remove();
      } // endif
    } // endif
  } // endwhile
  
  // Check for dying
  var dying = combatant.get("hp.dying.dying").booleanValue();
  if (dying)
    m = m + (m.length > 0 ? '<br/>' : '') + makeSavingThrow(combatant.get("hp"));
  return m;
}

// Make a saving throw and take into account any bonuses
//   keyword - The keyword for the save being made, allows bonuses for the keyword to be applied.
//   bonuses - The applied bonuses for saving throws.
//  Return: Text to display to user.
function makeSave(keyword, bonuses) {
  
  // Find applicable saving throw bonuses
  var j = bonuses.iterator();
  var kBonus = 0;
  var aBonus = 0;
  while (j.hasNext()) {
    var b = j.next();
    if (b.get('type') != 'Saving Throw') continue;
    if (String(b.get('keyword')) == String(keyword)) kBonus = b.get('amount').intValue();  
    if (String(b.get('keyword')) == 'Any') aBonus = b.get('amount').intValue();  
  } // endwhile
  var bonus = kBonus != 0 ? (aBonus != 0 ? Math.max(kBonus, aBonus) : kBonus) : aBonus;
  var roll = Math.floor(Math.random() * 20) + 1;
  if (bonus != 0) {
    var m = '1d20 + ' + bonus + ' &#8658; ' + roll + ' + ' + bonus + ' = ' + (roll + bonus) + '';
  } else {
    var m = '1d20 = ' + roll + '';
  } // endif
  if (roll + bonus >= 10) {
    m = m + " Success!";
  } else {
    m = m + " Failure!";
  } // endif
  return m;
}

/**
 * Get all of the macros for a combatant. This includes core, racial, class, and power macros
 * 
 * @param {PropertyMap} combatant Get all of this combatant's macros
 * @return {PropertyList} A list with all of the macros from the database in it.
 */ 
function getMacros(combatant) {
  
  var Java = new JavaImporter(net.rptools.chartool.model.property, net.rptools.chartool.model.db, 
                              net.rptools.lib, java.util);
  with (Java) {
    
    // Get core macros
    var macros = new PropertyList();
    var table = PropertyTable.getPropertyTable(PropertySettings.getInstance().getDatabaseName(), 'macro');
    var condition = new SimpleCondition('MACRO_SET_NAME', SimpleCondition.Operator.EQUALS, 'core')
    var list = table.selectRows(null, condition);
    if (!list.isEmpty()) macros.addAll(list);

    // Get any race macros
    var race = combatant.get('race.race');
    if (race != null && race.getName() != null) {
      condition = new SimpleCondition('MACRO_SET_NAME', SimpleCondition.Operator.EQUALS, 'race.' + race.getName().toLowerCase())
      list = table.selectRows(null, condition);
      if (!list.isEmpty()) macros.addAll(list);
    } // endif
    
    // Get any class macros
    var c = combatant.get('class.class');
    if (c != null && c.getName() != null) {
      condition = new SimpleCondition('MACRO_SET_NAME', SimpleCondition.Operator.EQUALS, 'class.' + c.getName().toLowerCase())
      list = table.selectRows(null, condition);
      if (!list.isEmpty()) macros.addAll(list);
    } // endif
    
    // Get all of the power macros
    var powers = combatant.get('powers.powers');
    var i = powers.iterator();
    while (i.hasNext()) {
      var power = i.next();
      condition = new SimpleCondition('MACRO_SET_NAME', SimpleCondition.Operator.EQUALS, 'power.' + power.getName())
      list = table.selectRows(null, condition);
      if (!list.isEmpty()) macros.addAll(list);
    } // endwhile
    return macros;
  } // endwith
}

/**
 * Get the brutal weapon value for any weapon.
 * 
 * @param {PropertyMap} weapon The map containing the weapon description.
 * @return {Number} The brutal value or 0 if the weapon is not brutal.
 */
function brutalWeapon(weapon) {
  var props = String(weapon.get("properties")).toLowerCase();
  var i = props.indexOf('brutal');
  if (i < 0) return 0;
  i += 6;
  props = props.substring(i);
  return parseInt(props);
}

/**
 * The names of all available equipment slots.
 */
var availableEquipmentSlots = ['armor', 'shield', 'weapon', 'offHand', 'arms', 'feet', 'hands', 'head', 'neck', 'leftRing', 
                               'rightRing', 'waist', 'holySymbol']

/**
 * Match up the equipped items to the items in the equipment list to ensure that they don't get out of synch. Only needed
 * when the character is first loaded.
 * 
 * @param {PropertyMap} combatant The combatant being modified. 
 */
function matchEquippedToItems(combatant) {
  
  // Make sure the item list is available
  if (!combatant || !combatant.containsKey('equipment.items')) return;
  var items = combatant.get('equipment.items');
  if (!items || items.isEmpty()) return;
  
  // Check each equipped item's value if it has one.
  for (var i = 0; i < availableEquipmentSlots.length; i++) {
    var key = 'equipped.' + availableEquipmentSlots[i];
    if (!combatant.containsKey(key)) continue;
    var e = combatant.get(key);
    if (e == null || e.isEmpty()) continue;
    var identifier = e.get('identifier');
    if (identifier == null || String(identifier).replace(/^\s+|\s+$/g, '').length == 0) continue;
    
    // Find the object in the items list
    var item = net.rptools.chartool.model.property.Utils.getListItem(items, identifier, 'identifier', null);
    if (!item) {
      print("Can't find a " + availableEquipmentSlots[i] + " slot item with identifier " + identifier + '\n');
      continue;
    } // endif
    if (e !== item) {
    	combatant.put(key, item);
    } // endif
  } // endif
}